/////////////////////////////////////////////////////////////////////////////
// Alfa.h : header file
// This class has a many functions:
//	- It generates the diplaylists for chars
//			CreateFont3D()		
//	- It handles preparation of text:
//			CalculateString(), GetEssayWidth(), GetEssayHeight()
//			SetText(), GetText()
//	- It acually draws the text:
//			Draw3DText()	
//	- It administrates change of font:
//			SetNewFont()
//	- It changes the "depth" of the text
//			SetExtrude()
//	- It can handle two different layouts (vertical and horizontal):
//			SetLayout();
//	- It can display the text as boxes
//			DrawAsText(),DrawAsBoxes(),IsAsBoxes()
//			
//-------------------------------------------------------------------
//	The text is drawn centered around origo
//	Vertical draw means that the text is drawn 
//	in a plane parallell to xy-plane, const z
//  Horizontal (nonvertical) means that the text is drawn
//	in a plane parallell to	xz-plane, const y
/////////////////////////////////////////////////////////////////////////////

#ifndef __ALFA_
#define __ALFA_

class CAlfa : public CCmdTarget
{
public:
	CAlfa();   // standard constructor

protected:
	// the text
	CString	m_Txt;
	// character depth
	GLfloat m_extrude;
	// the lines
	CStringArray m_TList; 
	// logical font
	LOGFONT m_lf;
	// keep lists
	GLYPHMETRICSFLOAT m_gmf[256];
	// start of fontlists
	int m_Start_List;
	// text metrics
	GLfloat m_leading, m_max_line_length;
	// character arrangement
	int m_bVertical;
	// characters may be drwan as boxes
	BOOL m_AsBox;


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAlfa)
	//}}AFX_VIRTUAL

// Implementation
public:
	void CreateFont3D();

	void CalculateString();	
	BOOL SetNewFont();

	void Draw3DText();

	GLfloat GetEssayWidth();
	GLfloat GetEssayHeight();

	void SetText(CString &S);
	void GetText(CString &S);

	void SetExtrude(GLfloat ext);
	GLfloat GetExtrude(){return m_extrude;}

	void SetLayout(BOOL bVertical=FALSE);
	BOOL GetLayout(){return m_bVertical;}

	void DrawAsText(){m_AsBox=FALSE;}
	void DrawAsBoxes(){m_AsBox=TRUE;}
	BOOL IsAsBoxes(){return m_AsBox;}

	// Generated message map functions
	//{{AFX_MSG(CAlfa)
	
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

};
#endif //__ALFA_
