// ////////////////////////////////////////////////////
//
// StdView.cpp : implementation of the CStdView class
// Based on class COGLView : public CView
// Inspect COGLView for standard OpenGL initialization
//
////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"
#include "mainfrm.h"
#include "math.h"
#include "readtxt.h"
#include "depth.h"

#include "StdDoc.h"
#include "StdView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////
// You can put definitions, variables and helperfunctions
// here. Functions that does not rely on CStdView member
// variables or functions

//////////////////////////////////////////////////////////
// CStdView

IMPLEMENT_DYNCREATE(CStdView, COGLView)

BEGIN_MESSAGE_MAP(CStdView, COGLView)
	//{{AFX_MSG_MAP(CStdView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(ID_SKRIFT_FONT, OnSkriftFont)
	ON_COMMAND(ID_SKRIFT_TEKST, OnSkriftTekst)
	ON_COMMAND(ID_VIEW_ANOTHERMATERIAL, OnViewAnothermaterial)
	ON_COMMAND(ID_VIEW_CLOSER, OnViewCloser)
	ON_COMMAND(ID_VIEW_FURTHER, OnViewFurther)
	ON_COMMAND(ID_VIEW_NORMAL, OnViewNormal)
	ON_COMMAND(ID_VIEW_VERTICALTEXT, OnViewVerticaltext)
	ON_COMMAND(ID_VIEW_ASBOXES, OnViewAsboxes)
	ON_UPDATE_COMMAND_UI(ID_VIEW_ASBOXES, OnUpdateViewAsboxes)
	ON_COMMAND(ID_VIEW_FOG, OnViewFog)
	ON_UPDATE_COMMAND_UI(ID_VIEW_FOG, OnUpdateViewFog)
	ON_UPDATE_COMMAND_UI(ID_VIEW_VERTICALTEXT, OnUpdateViewVerticaltext)
	ON_COMMAND(ID_VIEW_TEXTDEPTH, OnViewTextdepth)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////
// CStdView construction/destruction

CStdView::CStdView()
{
	SetHomePos();
	m_bFogOn=FALSE;
	m_hRC=NULL;
	m_hDC=NULL;

};

CStdView::~CStdView()
{}

////////////////////////////////////////////////////////////
// CStdView diagnostics
// You should usually not touch these:
#ifdef _DEBUG
void CStdView::AssertValid() const
{COGLView::AssertValid();}
void CStdView::Dump(CDumpContext& dc) const
{COGLView::Dump(dc);}

CStdDoc* CStdView::GetDocument() // non-debug version is inline
{	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CStdDoc)));
	return (CStdDoc*)m_pDocument;
}
#endif //_DEBUG

//////////////////////////////////////////////////////////////
// CStdView operations
void CStdView::SetHomePos()
{
	// initialize angles, see DrawScene
	m_Vz=0.0f;
	m_Vx=0.0f;
	m_Vy=0.0f;

	// Viewpoint
	m_EyeZ=6.0f;
	m_EyeY=m_EyeX=0.0f;

	// moving, animation
	m_dvy=0.0f;
	m_dvx=0.0f;
	m_bHasStopped=FALSE;
}


void CStdView::DrawScene()
{
	// Clear the color and depth buffers.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
	// Set up for scene
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	gluLookAt(	m_EyeX,m_EyeY,m_EyeZ,	
					0.0f,0.0f,0.0f,
					0.0f,1.0f,0.0f); // z up
	
	// tilt it around x
	glRotatef(m_Vx,1.0f,0.0f,0.0f);
	// tilt it around y
	glRotatef(m_Vy,0.0f,1.0f,0.0f);
	
	if(m_bFogOn)
	{
		///////////////////////////////
		// fog
		GLfloat fog_col[4]={1.0f,1.0f,1.0f,0.0f};
		glEnable(GL_FOG);
		glFogf(GL_FOG_MODE,GL_LINEAR);
		glFogfv(GL_FOG_COLOR,fog_col);
		glFogf(GL_FOG_END,7.0f);
		glFogf(GL_FOG_START,3.0f);
	}
	else
		glDisable(GL_FOG);

	/////////////////////////////
	// set material,
	m_MM.Do_Selection();

	glEnable(GL_NORMALIZE);

	/////////////////////////////
	// Draw the construction
	m_AA.Draw3DText();

	// finnish drawing
	glFlush();
}

void CStdView::InitializeRContext()
{
	// Here we set the attributes that will be constant
	// during the rendering contexts lifetime
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);
    
	// prepare ligthsource
	GLfloat ambient[] = {0.2f,0.2f,0.2f,1.0f };
    GLfloat diffuse[] = {1.0f,1.0f,1.0f,1.0f };
    GLfloat position[] = {4.0f,0.0f,6.0f,0.0f };
	GLfloat lmodel_ambient[] = {0.4f,0.4f,0.4f,1.0f };


    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
	
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);

	// smooth the drawing
	glShadeModel(GL_SMOOTH);

    // set background to light gray
	//glClearColor(0.8f, 0.8f, 0.8f, 0.0f);
	glClearColor(1.0f, 1.0f, 1.0f, 0.0f);

	// initialize the fontstuff
	m_AA.CreateFont3D();
}


void CStdView::SetPerspective(int cx, int cy) 
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	gluPerspective(50.0f, (GLdouble)cx/cy, 0.1f, 70.0f);
	glMatrixMode(GL_MODELVIEW);
}

void CStdView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// User has pressed left button.Make a note
	// of this and prepare for intuitive rotation of scene
	m_PLast=point;
	SetCapture();
	CView::OnLButtonDown(nFlags, point);
}

void CStdView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// free capture and terminate rotation
	ReleaseCapture();
	CView::OnLButtonUp(nFlags, point);
	
	if(m_bHasStopped)
	{
		// lower flag that indicates that this 
		// is the end of a stop click
		m_bHasStopped=FALSE;
	}
	else if((fabs(m_dvy)>1.0f)||(fabs(m_dvx)>1.0f))
	{
		// rotate around the x- and y-axis 
		// with degreesteps from interaction
		// stop rotation on mousedown
		MSG msg;
		do
		{
			m_Vy+=m_dvy;
			m_Vx+=m_dvx;
			RedrawWindow();
		}while (!PeekMessage(
					&msg,
					GetSafeHwnd(),
					WM_LBUTTONDOWN,
					WM_LBUTTONDOWN,
					PM_REMOVE)	// to make safer stop
					);
		// raise flag that indicates that we have had
		// a stop-click
		m_bHasStopped=TRUE;
	}
}

void CStdView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// if left button is down and this is not a result of
	// a stop-click
	if((nFlags & MK_LBUTTON)&&(!m_bHasStopped))
	{	
		// and has moved a significant dist
		if((abs(m_PLast.x-point.x)>2)||(abs(m_PLast.y-point.y)>2))
		{
			// then we find an intuitive rot angle
			CRect R;
			GetClientRect(&R);

			m_dvy=90.0f*((GLfloat (point.x-m_PLast.x))/GLfloat (R.Width()));
			m_dvx=90.0f*((GLfloat (point.y-m_PLast.y))/GLfloat (R.Height()));
			
			m_Vy+=m_dvy;
			m_Vx+=m_dvx;
			
			// rememeber point and force immediate redraw 
			m_PLast=point;
			RedrawWindow();
		}
	}
}

void CStdView::OnSkriftFont() 
{
	// let alfa-object give us a new font
	// time consuming since we must generate new 3D-font
	wglMakeCurrent(m_hDC,m_hRC);
	BOOL bRes=m_AA.SetNewFont();
	wglMakeCurrent(0,0);
	if(bRes)
		RedrawWindow();	
}

void CStdView::OnSkriftTekst() 
{
	// let alfa-object give us a new text
	CReadTxt rT;
	CString S;
	m_AA.GetText(S);
	rT.m_Txt=S;
	if(rT.DoModal()==IDOK)
	{
		S=rT.m_Txt;
		m_AA.SetText(S);
		RedrawWindow();
	}
}

void CStdView::OnViewAnothermaterial() 
{
	// User wants to change material of text
	if(m_MM.DoModal()==IDOK)
	{
		m_MM.Do_Selection(GL_FRONT);
		RedrawWindow();
	}
	
}

void CStdView::OnViewCloser() 
{
	// Move eye closer
	m_EyeZ-=0.5f;
	if(m_EyeZ<0.0f)
		m_EyeZ=0.1f;
	RedrawWindow();
}

void CStdView::OnViewFurther() 
{
	// Move eye closer
	m_EyeZ+=0.5f;
	RedrawWindow();
}

void CStdView::OnViewNormal() 
{
	// Reset view to were we staretd
	SetHomePos();
	RedrawWindow();
}

void CStdView::OnViewVerticaltext() 
{
	// Change layout
	m_AA.SetLayout(!m_AA.GetLayout());
	RedrawWindow();
}

void CStdView::OnUpdateViewVerticaltext(CCmdUI* pCmdUI) 
{
	// Is the text displayed vertical already
	pCmdUI->SetCheck(m_AA.GetLayout());
	
}

void CStdView::OnViewTextdepth() 
{
	// change char depth
	// time consuming since we must generate new 3D-font
	wglMakeCurrent(m_hDC,m_hRC);
	CDepth dD;
	dD.m_Depth=m_AA.GetExtrude();
	if(dD.DoModal()==IDOK)
		m_AA.SetExtrude(dD.m_Depth);
	wglMakeCurrent(0,0);
	RedrawWindow();
}


void CStdView::OnViewAsboxes() 
{
	if(m_AA.IsAsBoxes())
		m_AA.DrawAsText();
	else
		m_AA.DrawAsBoxes();
	RedrawWindow();
}

void CStdView::OnUpdateViewAsboxes(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_AA.IsAsBoxes());
}

void CStdView::OnViewFog() 
{
	m_bFogOn=!m_bFogOn;
	RedrawWindow();
}

void CStdView::OnUpdateViewFog(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_bFogOn);
}


